@@ -71,7 +71,17 @@ eDTOutput <- function(id,...) {
 #'   It will be evaluated for each new row in the environment defined by 'env'.
 #'   This allows for defaults like Sys.time() or uuid::UUIDgenerate() as well as dynamic inputs.
 #' @param env `environment` in which the server function is running. Should normally not be modified.
-#' 
+#' @inheritParams canXXXRowTemplate
+#' @param utilityColumns named character vector. Defines names for (hidden) utility columns
+#'   used by `eDT` to keep track of modifications. Should normally only be adjusted in rare case of name clashes with data.
+#'  ``` 
+#'  c(
+#'    status = '_editbl_status',
+#'    buttons = '_editbl_buttons',
+#'    identity = '_editbl_identity',
+#'    deleted = '_editbl_deleted'
+#'   )
+#'  ```
 #' @return list
 #' - result `reactive` modified version of `data` (saved)
 #' - state `reactive` current state of the `data` (unsaved)
@@ -113,6 +123,9 @@ eDTOutput <- function(id,...) {
 #'     ns("mpg"),
 #'     label = "mpg",
 #'     value = data$mpg)})
+#' 
+#'   # Do not allow delete
+#'   editbl::eDT(mtcars, canDeleteRow = FALSE)
 #' }
 #' 
 #' @author Jasper Schelfhout
@@ -152,8 +165,11 @@ eDT <- function(
     statusColor = c("insert"="#e6e6e6", "update"="#32a6d3", "delete"="#e52323"),
     inputUI = editbl::inputUI,
     defaults = tibble(),
-    env = environment()
-) {  
+    env = environment(),
+    canEditRow = TRUE,
+    canDeleteRow = TRUE,
+    utilityColumns = NULL
+) {
   args <- as.list(environment())
   
   # if not in reactive context start standalone app
@@ -222,14 +238,17 @@ eDTServer <- function(
     statusColor = c("insert"="#e6e6e6", "update"="#32a6d3", "delete"="#e52323"),
     inputUI = editbl::inputUI,
     defaults = tibble(),
-    env = environment()
-) {
+    env = environment(),
+    canEditRow = TRUE,
+    canDeleteRow = TRUE,
+    utilityColumns = NULL
+  ) {
   missingContainer <- missing(container)
   moduleServer(
       id,
       function(input, output, session) {
         ns <- session$ns
-                
+        
         rv <- reactiveValues(
             changelog = list(),
             changeLogTracker = 0,
@@ -238,6 +257,19 @@ eDTServer <- function(
             changelog_react = 0 # # to force refreshing even when reactive value stays the same
         )
         
+        # Utility columns
+        defaultUtilityColumns <- c(
+            status = '_editbl_status',
+            buttons = '_editbl_buttons',
+            identity = '_editbl_identity',
+            deleted = '_editbl_deleted'
+        )
+        utilityColumns <- overwriteDefaults(defaultUtilityColumns, utilityColumns)
+        statusCol <- utilityColumns[['status']]
+        buttonCol <- utilityColumns[['buttons']]
+        identityCol <- utilityColumns[['identity']]
+        deleteCol <- utilityColumns[['deleted']]
+        
         # Make arguments reactive / set defaults
         # This way users can pass on both reactive an non reactive arguments
         # Need to be explicit about environement. Otherwhise they overwrite themselves.
@@ -350,7 +382,7 @@ eDTServer <- function(
         if(!shiny::is.reactive(inputUI)){
           inputUI <- shiny::reactive(inputUI, env = argEnv)
         }
-
+        
         if(!shiny::is.reactive(defaults)){  
           defaults <- shiny::reactive({
                 eval(substitute(defaults, env))
@@ -358,6 +390,14 @@ eDTServer <- function(
               env = argEnv)
         }
         
+        if(!shiny::is.reactive(canEditRow)){
+          canEditRow <- shiny::reactive(canEditRow, env = argEnv)
+        }
+        
+        if(!shiny::is.reactive(canDeleteRow)){
+          canDeleteRow <- shiny::reactive(canDeleteRow, env = argEnv)
+        }
+        
         # Force re-evaluting reactive for values like Sys.time(), uuid::UUIDgenerate()
         defaultsAddBound <- defaults %>% shiny::bindEvent(input$add)
         
@@ -399,17 +439,23 @@ eDTServer <- function(
                 data <- joinForeignTbl(data,foreignTbl)
               }
               
-              # FIXME: support these column names
-              utilityColumns <- c("buttons", "i", "status", "deleted")
               if(any(utilityColumns %in% dplyr::tbl_vars(data))){
-                stop(sprintf("Due to current code design, `editbl` does not support any of following column names: %s. \n Will be fixed on a next release.",
+                stop(sprintf("Adjust the utility columns such that they don't clash with the data column names.",
                         paste(utilityColumns, collapse = ", ")))
               }
               
               data <- dplyr::collect(data)
               data <- as.data.frame(data)
               
-              data <- initData(data, ns = ns)
+              data <- initData(
+                  data,
+                  ns = ns,
+                  canEditRow = isolate(canEditRow()),
+                  canDeleteRow = isolate(canDeleteRow()),
+                  statusCol = statusCol,
+                  buttonCol = buttonCol,
+                  iCol = identityCol,
+                  deleteCol = deleteCol)
               rv$checkPointData <- data
               rv$modifiedData <- data
               rv$changelog <- list()
@@ -425,7 +471,7 @@ eDTServer <- function(
               req(!is.null(rv$newState) && isTruthy(rv$newState))
               castCols <- base::colnames(isolate(data()))
               data <- rv$newState
-              data <- relocate(data,  dplyr::all_of("buttons"))     
+              data <- relocate(data,  dplyr::all_of(buttonCol))     
               rv$modifiedData <- data
               
               data <- castForDisplay(data, cols = castCols)
@@ -451,14 +497,14 @@ eDTServer <- function(
               
               data <- isolate(rv$modifiedData)
               
-              baseCols <- setdiff(base::colnames(data), c("buttons", "i", "status", "deleted"))
+              baseCols <- setdiff(base::colnames(data), utilityColumns)
               baseColsI <- which(base::colnames(data) %in% baseCols)
-              buttonCol <- which(base::colnames(data) == "buttons") -1 + rownames
+              buttonColI <- which(base::colnames(data) == buttonCol) -1 + rownames
               deductedCols <- which(base::colnames(data) %in% deductedColnames()) -1 + rownames
               
               data <- castForDisplay(data)
               
-              bcol <- "buttons"
+              bcol <- buttonCol
               names(bcol) <- ""
               colnames <- c(bcol,colnames)
               
@@ -468,20 +514,20 @@ eDTServer <- function(
                   list(list(
                           visible = FALSE,
                           targets = which(base::colnames(data) %in% 
-                                  c("status", "i", "deleted")) - !rownames)
+                                  c(statusCol, identityCol, deleteCol)) - !rownames)
                   ))
               
               if(escape == TRUE){
-                escape <- -buttonCol
+                escape <- -buttonColI
               }
               else if(escape == FALSE){
                 escape <- FALSE
               }
               else if(is.numeric(escape)){
-                escape <- c(escape, -buttonCol)
+                escape <- c(escape, -buttonColI)
               } else if(is.character(escape)){
                 escape <- which(base::colnames(data) %in% escape) + rownames
-                escape <- c(escape, -buttonCol)
+                escape <- c(escape, -buttonColI)
               }
               
               # Make sure utility columns are not editable
@@ -489,10 +535,10 @@ eDTServer <- function(
               
               if(!inherits(editable, "logical")){
                 if(!"disable" %in% names(editable)){
-                  editable <- c(editable, list("disable" = list("columns" = c(buttonCol, deductedCols))))
+                  editable <- c(editable, list("disable" = list("columns" = c(buttonColI, deductedCols))))
                 } else {
                   editable$disable <- list("columns" = unique(c(editable$disable$columns,
-                              buttonCol,
+                              buttonColI,
                               deductedCols)))
                 }
               }
@@ -514,21 +560,21 @@ eDTServer <- function(
                           "redo" = icon("rotate-right"),
                           "save" = icon("floppy-disk"),
                           ""
-                          )   
+                      )   
                       disabled = switch(x,
-                              "add" = FALSE,
-                              "undo" = TRUE,
-                              "redo" =  TRUE,
-                              "save" = TRUE,
-                              TRUE
-                          )       
-                          
+                          "add" = FALSE,
+                          "undo" = TRUE,
+                          "redo" =  TRUE,
+                          "save" = TRUE,
+                          TRUE
+                      )       
+                      
                       customButton(ns(x), label = x, icon, disabled = disabled)
                     } else {
                       x
                     }
                   })
-                            
+              
               # Deal with the fact that 'container' can be a missing argument
               # Which is why put arguments in a list and use do.call instead of passing on directly.
               # FIXME: there should be a better approach              
@@ -558,11 +604,11 @@ eDTServer <- function(
               }
               
               do.call(DT::datatable, internalArgs) %>%
-                  formatStyle("status", target='row',
+                  formatStyle(statusCol, target='row',
                       backgroundColor = styleEqual('inserted',statusColor()["insert"]))%>%
-                  formatStyle("status", target='row',
+                  formatStyle(statusCol, target='row',
                       backgroundColor = styleEqual('edited',statusColor()["update"]))%>%
-                  formatStyle("deleted", target='row',
+                  formatStyle(deleteCol, target='row',
                       backgroundColor = styleEqual(TRUE,statusColor()["delete"])) %>%
                   format()()
             })
@@ -571,7 +617,6 @@ eDTServer <- function(
         
         notInModalColumns <- reactive({              
               # Columns that should not be edited through the modal
-              status <- c("i", "buttons", "status", "deleted")
               deducted <- deductedColnames() # Therefore non-editable
               invisible <- unlist(lapply(options()$columnDefs, function(x){
                         if(!is.null(x$visible)){
@@ -582,7 +627,7 @@ eDTServer <- function(
                       }))
               notEditable <- names(rv$modifiedData)[editable_std()$disable$columns + 1]
               unique(c(
-                      status,
+                      utilityColumns,
                       deducted,
                       invisible,
                       notEditable
@@ -618,15 +663,16 @@ eDTServer <- function(
             })
         
         clickedRow <- reactive({
-              as.numeric(sub("^.*_","",input$current_id))
+              identity = sub("^.*_","",input$current_id)
+              which(rv$modifiedData[,identityCol] == identity)
             })
         
         effectiveChanges <- reactive({
               data <- do.call(rbind, rv$changelog[seq_len(rv$changeLogTracker)])
               
               # get last state of the row per id
-              data <- do.call(rbind,(lapply(unique(data$i), 
-                            function(i){tail(data[data$i == i,],1)})))
+              data <- do.call(rbind,(lapply(unique(data[[identityCol]]), 
+                            function(i){tail(data[data[[identityCol]] == i,],1)})))
               data
             })
         
@@ -641,17 +687,18 @@ eDTServer <- function(
               i <- rv$changeLogTracker
               req(i > 0)
               data <- rv$modifiedData
-              data$buttons <- NULL
               undoChanges <- rv$changelog[[rv$changeLogTracker]]
               
               for(row in seq_len(nrow(undoChanges))){
                 undoChange <- undoChanges[row,]
                 
                 lastLogState <- do.call(rbind,rv$changelog[seq_len(max(0,i-1))])
-                lastLogState <- tail(lastLogState[lastLogState$i == undoChange$i,],1)
+                lastLogState <- tail(
+                    lastLogState[lastLogState[[identityCol]] == undoChange[[identityCol]],],1
+                )
                 
-                lastCheckPointState <- rv$checkPointData[rv$checkPointData$i == undoChange$i,]
-                lastCheckPointState$buttons <- NULL
+                lastCheckPointState <- rv$checkPointData[
+                    rv$checkPointData[[identityCol]] == undoChange[[identityCol]],]
                 
                 if(!is.null(lastLogState) && nrow(lastLogState)){
                   stateBeforeChange <- lastLogState
@@ -659,17 +706,16 @@ eDTServer <- function(
                   stateBeforeChange <- lastCheckPointState
                 }
                 
-                if(undoChange$status == "inserted" && nrow(stateBeforeChange) == 0){ # delete if row did not exist before
-                  data <- data[data$i != undoChange$i,]
-                } else if (undoChange$status == "deleted" && !undoChange$i %in% data$i){ # re-insert if row is now deleted
+                if(undoChange[[statusCol]] == "inserted" && nrow(stateBeforeChange) == 0){ # delete if row did not exist before
+                  data <- data[data[[identityCol]] != undoChange[[identityCol]],]
+                } else if (undoChange[[statusCol]] == "deleted" && !undoChange[[identityCol]] %in% data[[identityCol]]){ # re-insert if row is now deleted
                   data <- cbind(undoChange, data)
                 } else { # set row to previous state
-                  data[data$i == undoChange$i,] <- stateBeforeChange
+                  data[data[[identityCol]] == undoChange[[identityCol]],] <- stateBeforeChange
                 }
               }
               
               rv$changeLogTracker <- max(0, rv$changeLogTracker - 1)
-              data <- addButtons(data, "buttons", ns)
               rv$newState <- data
             })
         
@@ -679,21 +725,19 @@ eDTServer <- function(
               req(i < nChanges)
               
               data <- rv$modifiedData
-              data$buttons <- NULL
               
               redoChanges <- rv$changelog[[i + 1]]
               
               for (iRedoChange in seq_len(nrow(redoChanges))){
                 redoChange <- redoChanges[iRedoChange,]
-                if(redoChange$status == "inserted" && !redoChange$i %in% data$i){
+                if(redoChange[[statusCol]] == "inserted" && !redoChange[[identityCol]] %in% data[[identityCol]]){
                   data <- rbind(redoChange,data)
                 } else {
-                  data[data$i == redoChange$i,] <- redoChange
+                  data[data[[identityCol]] == redoChange[[identityCol]],] <- redoChange
                 }
               }
               
               rv$changeLogTracker <- i + 1
-              data <- addButtons(data, "buttons", ns)
               rv$newState <- data
               
             })
@@ -701,15 +745,18 @@ eDTServer <- function(
         observeEvent(input$confirmEdit, {
               i <- clickedRow()
               data <- rv$modifiedData
+              req(evalCanEditRow(
+                      row=data[i,],
+                      canEditRow=canEditRow(),
+                      statusCol=statusCol))
               data[i,] <-  fillDeductedColumns(rv$modalData(), foreignTbls())
               
-              currentStatus <- data[i,"status"]
+              currentStatus <- data[i,statusCol]
               if(currentStatus == "unmodified"){
-                data[i,"status"] <- "edited"
+                data[i,statusCol] <- "edited"
               }
               
               newChange <- data[i,]
-              newChange$buttons <- NULL
               changelog <- rv$changelog[seq_len(rv$changeLogTracker)]
               changelog[[rv$changeLogTracker +1]] <- newChange
               rv$changelog <- changelog
@@ -730,7 +777,7 @@ eDTServer <- function(
               rv$edits <- input$DT_cell_edit
               rv$edits_react <-  rv$edits_react + 1
             })
-         
+        
         observeEvent(rv$edits_react, {
               req(!is.null(rv$edits) && isTruthy(rv$edits))
               edits <- unique(rv$edits)
@@ -743,6 +790,15 @@ eDTServer <- function(
                     for(i in sort(unique(edits$row))){
                       changes <- edits[edits$row == i,]
                       currentRow <- data[i,]
+                      if(!evalCanEditRow(
+                          row = currentRow, 
+                          canEditRow=canEditRow(), 
+                          statusCol=statusCol
+                          )
+                      ){
+                        next
+                      }
+                      
                       newRow <- currentRow
                       
                       hasChanged <- FALSE
@@ -757,9 +813,9 @@ eDTServer <- function(
                         if(!identical(currentValue, newValue)){
                           hasChanged <- TRUE
                           newRow[,j] <- newValue
-                          currentStatus <- newRow[,"status"]
+                          currentStatus <- newRow[[statusCol]]
                           if(currentStatus == "unmodified"){
-                            newRow[,"status"] <- "edited"
+                            newRow[,statusCol] <- "edited"
                           }
                         }
                       }
@@ -775,13 +831,14 @@ eDTServer <- function(
                     }    
                     newChanges <- do.call(rbind, newRows)
                     
-                    newChanges$buttons <- NULL
                     changelog <- rv$changelog[seq_len(rv$changeLogTracker)]
                     changelog[[rv$changeLogTracker +1]] <- newChanges
                     rv$changelog <- changelog
                     rv$changelog_react <- rv$changelog_react + 1
                     
                     rv$newState <- data
+                    rv$triggerNewState <- Sys.time() # Need to reupdate if edits were not allowed
+                    
                   },
                   error = function(e){
                     rv$resultMessage <- "The change you just made is not allowed. Reverting."
@@ -796,10 +853,15 @@ eDTServer <- function(
               rowNumber <- clickedRow()
               data <- rv$modifiedData
               row <- data[rowNumber,]
-              row$deleted <- !row$deleted
+              req(evalCanDeleteRow(
+                      row = row,
+                      canDeleteRow=canDeleteRow(),
+                      statusCol=statusCol
+                  )
+              )
+              row[[deleteCol]] <- !row[[deleteCol]]
               
               newChange <- row
-              newChange$buttons <- NULL
               changelog <- rv$changelog[seq_len(rv$changeLogTracker)]
               changelog[[rv$changeLogTracker +1]] <- newChange
               rv$changelog <- changelog
@@ -810,8 +872,6 @@ eDTServer <- function(
         
         observeEvent(input$add,{
               data <- rv$modifiedData
-              data$buttons <- NULL
-                            
               # create new row
               newRow <- data %>%
                   dplyr::filter(FALSE)
@@ -831,9 +891,16 @@ eDTServer <- function(
                   newRow[[col]] <- defaults[[col]]
                 }
               }
-              newRow$status <- "inserted"
-              newRow$deleted <- FALSE
-              newRow$i <- uuid::UUIDgenerate()
+              newRow[,statusCol] <- "inserted"
+              newRow[,deleteCol] <- FALSE
+              newRow[,identityCol] <- uuid::UUIDgenerate()
+              newRow <- addButtons(
+                  df = newRow,
+                  columnName = buttonCol,
+                  iCol = identityCol,
+                  ns = ns,
+                  canDeleteRow = canDeleteRow(),
+                  canEditRow = canEditRow())
               
               # save to changelog
               newChange <- newRow
@@ -846,30 +913,28 @@ eDTServer <- function(
                   data
               )
               
-              data <- addButtons(data, "buttons", ns = ns)
-              
               rv$newState <- data
             })
         
         effectiveInserted <- reactive({
               modified <- effectiveChanges()         
               modified[
-                  modified$status == "inserted" &
-                      modified$deleted == FALSE,]
+                  modified[[statusCol]] == "inserted" &
+                      modified[[deleteCol]] == FALSE,]
             })
         
         effectiveEdited <- reactive({
               modified <- effectiveChanges()
               modified[
-                  modified$status == "edited" &
-                      modified$deleted == FALSE,]
+                  modified[[statusCol]] == "edited" &
+                      modified[[deleteCol]] == FALSE,]
             })
         
         effectiveDeleted <- reactive({
               modified <- effectiveChanges()
               modified[
-                  modified$status != "inserted" &
-                      modified$deleted == TRUE,]
+                  modified[[statusCol]] != "inserted" &
+                      modified[[deleteCol]] == TRUE,]
             })
         
         observeEvent(input$save,{
@@ -930,22 +995,22 @@ eDTServer <- function(
                     
                     # deletes
                     deleted <- merge(
-                        modified[modified$deleted == TRUE,"i",drop = FALSE],
+                        modified[modified[[deleteCol]] == TRUE,identityCol,drop = FALSE],
                         checkPoint,
-                        by = "i")[,keys(),drop = FALSE]
+                        by = identityCol)[,keys(),drop = FALSE]
                     
                     # edits
                     edited <- effectiveEdited()
                     if(!checkForeignTbls(edited, foreignTbls())){
                       stop("You made invalid edits to a row.")
                     }
-                    edited <- edited[, c(cols, "i")]
+                    edited <- edited[, c(cols, identityCol)]
                     match_x <-  merge(
-                        edited[,"i", drop = FALSE],
+                        edited[,identityCol, drop = FALSE],
                         checkPoint,
-                        by = "i")
-                    match_x <- match_x[order(match_x$i), keys(), drop = FALSE]
-                    match_y <- edited[order(edited$i), keys(), drop = FALSE]
+                        by = identityCol)
+                    match_x <- match_x[order(match_x[[identityCol]]), keys(), drop = FALSE]
+                    match_y <- edited[order(edited[[identityCol]]), keys(), drop = FALSE]
                     match <- list(
                         x = match_x,
                         y = match_y
@@ -1033,13 +1098,31 @@ eDTServer <- function(
                       rv$fullTableRefresh <- UUIDgenerate()
                     } else {
                       checkPointState <- rv$modifiedData
-                      checkPointState <- checkPointState[checkPointState$deleted != TRUE,]
-                      checkPointState$status <- 'unmodified'
-                      checkPointState$deleted <- FALSE
-                      checkPointState$buttons <- NULL
-                      rv$changelog <- list()
-                      checkPointState <- addButtons(checkPointState, "buttons", ns)
                       
+                      # Remove deleted rows
+                      checkPointState <- checkPointState[checkPointState[[deleteCol]] != TRUE,]
+                      
+                      # Update inserted rows to being 'unmodified'
+                      insertedRows <- which(checkPointState[,statusCol] == 'inserted')
+                      for(i in insertedRows){
+                        currentRow <- checkPointState[i,]
+                        adjustedRow <- currentRow
+                        adjustedRow[,statusCol] <- 'unmodified'
+                        adjustedRow <- addButtons(
+                            df = adjustedRow,
+                            columnName = buttonCol,
+                            iCol = identityCol,
+                            ns = ns,
+                            canEditRow = canEditRow(),
+                            canDeleteRow = canDeleteRow()
+                            )
+                        checkPointState[i,] <- adjustedRow
+                      }
+                      
+                      # Update edited rows to being 'unmodified'
+                      checkPointState[[statusCol]] <- 'unmodified'
+
+                      rv$changelog <- list()
                       rv$checkPointData <- checkPointState
                       rv$newState <- checkPointState
                     }
@@ -1078,16 +1161,16 @@ eDTServer <- function(
         observe(priority = 1,{
               req(!is.null(rv$modifiedData) && isTruthy(rv$modifiedData))
               req(!is.null(isolate(rv$selected)) && isTruthy(isolate(rv$selected)))
-
-              currentSelection <- isolate(rv$selected)$i  
-              newIndexes <- which(rv$modifiedData$i %in% currentSelection)
+              
+              currentSelection <- isolate(rv$selected)[[identityCol]]  
+              newIndexes <- which(rv$modifiedData[[identityCol]] %in% currentSelection)
               if(length(newIndexes)){
                 DT::selectRows(proxyDT, newIndexes)
               }
             })
         
         observe({
-               rv$selected <- selected() # To force evaluation
+              rv$selected <- selected() # To force evaluation
             })
         
         selected <- reactive({
@@ -1099,16 +1182,92 @@ eDTServer <- function(
         dataVars <- reactive({
               dplyr::tbl_vars(data())
             })
-                
+        
         return(list(
                 result = result,
                 state = reactive({castToTemplate(rv$modifiedData[,dataVars()], data())}),
                 selected = reactive({castToTemplate(selected()[,dataVars()], data())})
-                ))
+            ))
       }
   )
 }
 
+#' Determine if a row can be deleted
+#' 
+#' @details calling this around the user passed on function ensures
+#'  that newly inserted rows are being excempt from the logic.
+#'  Moreover, the output of the function can be checked.
+#' 
+#' @param row `tibble`, single row
+#' @param canDeleteRow `function` with argument 'row' defining logic on wether or
+#'    not the row can be modified. Can also be `logical` TRUE or FALSE.
+#' @param statusCol `character(1)` name of column with general status (e.g. modified or not).
+#' @return `boolean`
+#' 
+#' @author Jasper Schelfhout
+evalCanDeleteRow <- function(
+    row,
+    canDeleteRow = TRUE,
+    statusCol='status'
+  ){
+  # Prevent evaluating logic and speed up for most common use-case
+  if(is.logical(canDeleteRow) && canDeleteRow){
+    return(TRUE)
+  }
+  
+  if (!is.null(statusCol) && row[[statusCol]] == 'inserted'){
+    return(TRUE)
+  }
+  
+  if(is.function(canDeleteRow)){
+    result <- canDeleteRow(row=row)
+    if(!is.logical(result)){
+      stop('canDeleteRow should return a logical value.')
+    }
+  } else if (is.logical(canDeleteRow)) {
+    result <- canDeleteRow
+  }
+  
+  return(result)
+}
+
+#' Determine if a row can be edited
+#' 
+#' @details calling this around the user passed on function ensures
+#'  that newly inserted rows are being excempt from the logic.
+#'  Moreover, the output of the function can be checked.
+#' 
+#' @param row `tibble`, single row.
+#' @param canEditRow `function` with argument 'row' defining logic on wether or
+#'    not the row can be modified. Can also be `logical` TRUE or FALSE.
+#' @param statusCol `character(1)` name of column with general status (e.g. modified or not).
+#' @return `boolean`
+#' 
+#' @author Jasper Schelfhout
+evalCanEditRow <- function(row, canEditRow = TRUE, statusCol='status'){
+  
+  # Prevent evaluating logic and speed up for most common use-case
+  if(is.logical(canEditRow) && canEditRow){
+    return(TRUE)
+  }
+  
+  if (!is.null(statusCol) && row[[statusCol]] == 'inserted'){
+    return(TRUE)
+  }
+  
+  if(is.function(canEditRow)){
+    result <- canEditRow(row=row)
+    if(!is.logical(result)){
+      stop('canEditRow should return a logical value.')
+    }
+  } else if (is.logical(canEditRow)) {
+    result <- canEditRow
+  }
+  
+  return(result)
+}
+
+
 #' Add some extra columns to data to allow for / keep track of modifications
 #' @param data `data.frame`
 #' @param ns namespace function
@@ -1116,6 +1275,7 @@ eDTServer <- function(
 #' @param statusCol `character(1)` name of column with general status (e.g. modified or not).
 #' @param deleteCol `character(1)` name of the column with deletion status.
 #' @param iCol `character(1)` name of column containing a unique identifier.
+#' @inheritParams canXXXRowTemplate
 #' @return data with extra columns buttons, status, i.
 #' @importFrom dplyr relocate all_of
 #' @importFrom uuid UUIDgenerate
@@ -1126,7 +1286,9 @@ initData <- function(
     buttonCol = "buttons",
     statusCol = "status",
     deleteCol = "deleted",
-    iCol = "i"
+    iCol = "i",
+    canDeleteRow = TRUE,
+    canEditRow = TRUE
 ){
   data[statusCol] <- rep("unmodified", nrow(data))
   data[deleteCol] <- rep(FALSE, nrow(data))
@@ -1139,7 +1301,11 @@ initData <- function(
   data <- addButtons(
       df = data,
       columnName = buttonCol,
-      ns = ns)
+      ns = ns,
+      iCol = iCol,
+      canEditRow = canEditRow,
+      canDeleteRow = canDeleteRow,
+      statusCol=NULL) # Not to trigger unneeded logic since all values are 'unmodified'
   data <- relocate(data, all_of(buttonCol))
   data
 }
@@ -1148,67 +1314,182 @@ initData <- function(
 #' @param df `data.frame`
 #' @param columnName `character(1)`
 #' @param ns namespace function
+#' @param iCol `character(1)` name of column containing a unique identifier.
+#' @param statusCol `character(1)` name of column with general status (e.g. modified or not).
+#'    if `NULL`, the data is interpreted as 'unmodified'.
+#' @inheritParams canXXXRowTemplate
 #' @return df with extra column containing buttons
+#' @importFrom dplyr across everything rowwise
+#' @importFrom rlang :=
 #' 
 #' @author Jasper Schelfhout
-addButtons <- function(df, columnName, ns){
+addButtons <- function(
+    df,
+    columnName,
+    ns,
+    iCol = 'i',
+    canEditRow = TRUE,
+    canDeleteRow = TRUE,
+    statusCol = 'status'
+){
+  ns_char = ns("")
+  
   if(!nrow(df)){
     df[columnName] <- character(0)
     return(df)
+  } else {
+    df <- df %>%
+        dplyr::rowwise() %>%
+        dplyr::mutate(!!columnName := createButtons(
+                row = dplyr::across(dplyr::everything(), ~ .),
+                suffix = get(iCol),
+                ns = !!ns_char,
+                canEditRow = !!canEditRow,
+                canDeleteRow = !!canDeleteRow,
+                statusCol = !!statusCol
+            )) %>%
+        dplyr::ungroup() %>%
+        as.data.frame()
   }
   
-  df[columnName] <- lapply(seq_len(nrow(df)), function(i){
-        createButtons(i, ns)
-      }) %>% unlist()
-  df     
+  df
 }
 
+
 #' Helper function to write HTML
-#' @details generate HTML as character once and reuse.
-#' Since buttons have to be generated a lot, this otherwhise slows down the app.
-#' @param suffix `character(1)` sprintf placeholer for suffix
-#' @param ns `character(1)` sprintf placeholder for ns
+#' @inheritParams createDeleteButtonHTML
+#' @details only to be used interactively. sprintf() implementation
+#'   is faster.
 #' @importFrom shiny div actionButton icon
-#' @return `character(1)` HTML to be filled in with \code{sprintf}
-createButtonsHTML <- function(suffix = "%1$s", ns = "%2$s"){
+#' @seealso createEditButtonHTML
+createDeleteButtonHTML_shiny <- function(
+    ns = "%1$s",
+    suffix = "%2$s",
+    disabled = FALSE){
   as.character(
-      div(class = "btn-group",
-          actionButton(
-              inputId =  paste0(ns, "delete_row_", suffix),
-              label = "",
-              icon = icon("trash"),
-              style = "color: red;background-color: white",
-              onclick =  HTML(sprintf("get_id(this.id, '%1$s');
-                          Shiny.setInputValue(\"%1$sdelete\", Math.random(), {priority: \"event\"});",
-                      ns))
-          ),
-          actionButton(
-              inputId =  paste0(ns, "edit_row_", suffix),
-              label = "",
-              icon = icon("pen-to-square"),
-              style = "background-color: white",
-              onclick =  HTML(sprintf("get_id(this.id, '%1$s');
-                          Shiny.setInputValue(\"%1$sedit\", Math.random(), {priority: \"event\"});",
-                      ns))
+      actionButton(
+          inputId = sprintf("%1$sdelete_row_%2$s", ns,suffix),
+          label = "",
+          icon = icon("trash"),
+          style = "color: red;background-color: white",
+          onclick =  HTML(sprintf("get_id(this.id, '%1$s');
+                      Shiny.setInputValue(\"%1$sdelete\", Math.random(), {priority: \"event\"});",
+                  ns
+              )
           )
       )
   )
 }
 
-buttonsHTML <- createButtonsHTML()
 
-#' Create buttons to modify the row. See \code{\link{createButtonsHTML}}
+#' Helper function to write HTML
+#' @inheritParams createEditButtonHTML
+#' @details only to be used interactively. sprintf() implementation
+#'   is faster.
+#' @seealso createEditButtonHTML
+#' @importFrom shiny div actionButton icon
+createEditButtonHTML_shiny <- function(
+    ns = "%1$s",
+    suffix = "%2$s",
+    disabled = FALSE){
+  as.character(
+      actionButton(
+          inputId = sprintf("%1$sedit_row_%2$s", ns,suffix),
+          label = "",
+          disabled = disabled,
+          icon = icon("pen-to-square"),
+          style = "background-color: white",
+          onclick = HTML(sprintf("get_id(this.id, '%1$s');
+                      Shiny.setInputValue(\"%1$sedit\", Math.random(), {priority: \"event\"});",
+                  ns
+              ))
+      )  
+  )
+}
+
+#' Generate HTML for an in-row edit button
+#' @param suffix `character(1)` id of the row
+#' @param ns `character(1)` namespace
+#' @param disabled `logical(1)` wether or not the button has to be disabled
+#' @return `character(1)` HTML
+createEditButtonHTML <- function(
+    ns,
+    suffix,
+    disabled = FALSE
+){
+  if(disabled){
+    disabled_str = 'disabled'
+  } else {
+    disabled_str = ''
+  }
+  sprintf(r"(<button id="%1$sedit_row_%2$s" type="button" class="btn btn-default action-button" %3$s style="background-color: white" onclick="get_id(this.id, &#39;%1$s&#39;);&#10;                      Shiny.setInputValue(&quot;%1$sedit&quot;, Math.random(), {priority: &quot;event&quot;});">
+      <i class="far fa-pen-to-square" role="presentation" aria-label="pen-to-square icon"></i>   
+      </button>)", ns, suffix, disabled_str)
+}
+
+#' Generate HTML for an in-row delete button
+#' @param suffix `character(1)` id of the row
+#' @param ns `character(1)` namespace
+#' @param disabled `logical(1)` wether or not the button has to be disabled
+#' @return `character(1)` HTML
+createDeleteButtonHTML <- function(
+    ns = "%1$s",
+    suffix = "%2$s",
+    disabled=FALSE){
+  if(disabled){
+    disabled_str = 'disabled'
+  } else {
+    disabled_str = ''
+  }
+  sprintf(r"(<button id="%1$sdelete_row_%2$s" type="button" class="btn btn-default action-button"  %3$s style="color: red;background-color: white" onclick="get_id(this.id, &#39;%1$s&#39;);&#10;                      Shiny.setInputValue(&quot;%1$sdelete&quot;, Math.random(), {priority: &quot;event&quot;});">
+      <i class="fas fa-trash" role="presentation" aria-label="trash icon"></i>
+      </button>)", ns, suffix, disabled_str)
+}
+
+
+#' Re-usable documentation
+#' @param canEditRow can be either of the following:
+#'    - `logical`, e.g. TRUE or FALSE
+#'    - `function`. Needs as input an argument `row` which accepts a single row `tibble` and as output TRUE/FALSE.
+#' @param canDeleteRow can be either of the following:
+#'    - `logical`, e.g. TRUE or FALSE
+#'    - `function`. Needs as input an argument `row` which accepts a single row `tibble` and as output TRUE/FALSE.
+canXXXRowTemplate <- function(canEditRow, canDeleteRow){
+  NULL
+}
+
+#' Create buttons to modify the row.
 #' @details buttons used per row in the app.
+#' @param row `tibble` with single row
 #' @param suffix `character(1)`
 #' @param ns `character(1)` namespace
-#' @return `character` HTML
-createButtons <- function(suffix, ns){
-  sprintf(
-      # Can be generated with createButtonsHTML
-      buttonsHTML,
-      suffix,
-      ns("")
+#' @param statusCol `character(1)` name of column with general status (e.g. modified or not).
+#'    if `NULL`, the data is interpreted as 'unmodified'.
+#' @inheritParams canXXXRowTemplate
+#' @return `character(1)` HTML
+createButtons <- function(
+    row,
+    suffix,
+    ns, 
+    canEditRow = TRUE,
+    canDeleteRow = TRUE,
+    statusCol = 'status'
+){
+  deleteButton <- createDeleteButtonHTML(
+      ns=ns,
+      suffix=suffix,
+      disabled = !evalCanDeleteRow(row=row, canDeleteRow=canDeleteRow, statusCol=statusCol))
+  editButton <- createEditButtonHTML(
+       ns=ns,
+       suffix=suffix,
+       disabled = !evalCanEditRow(row=row, canEditRow=canEditRow, statusCol=statusCol))
+
+  result <- sprintf('<div class="btn-group">%1$s%2$s</div>',
+      deleteButton,
+      editButton
   )
+  
+  result
 }
 
 #' Function to generate CSS to disable clicking events on a column
@@ -1218,10 +1499,10 @@ createButtons <- function(suffix, ns){
 #' @return `character` CSS
 disableDoubleClickButtonCss <- function(id){
   sprintf("
-      #%1$s > .dataTables_wrapper > table tbody td:nth-child(1) {pointer-events: none;}
-      #%1$s > .dataTables_wrapper > table tbody td:nth-child(1)>div {pointer-events: auto;}
-      ",id)
-} 
+          #%1$s > .dataTables_wrapper > table tbody td:nth-child(1) {pointer-events: none;}
+          #%1$s > .dataTables_wrapper > table tbody td:nth-child(1)>div {pointer-events: auto;}
+          ",id)
+}
 
 keyTableJS <- c(
     # Trigger doubleclick by enter
